@@ -674,7 +674,7 @@ void DebugSession::slotProgramStopped(const GDBMI::ResultRecord& r)
     if (reason == "exited-normally" || reason == "exited")
     {
         if (r.hasField("exit-code")) {
-            programNoApp(i18n("Exited with return code: %1").arg(r["exit-code"].literal()));
+            programNoApp(i18n("Exited with return code: %1", r["exit-code"].literal()));
         } else {
             programNoApp(i18n("Exited normally"));
         }
