@@ -26,22 +26,22 @@ class QStringPrinter:
         self.val = val
 
     def to_string(self):
-        #ret = ""
-        #i = 0
         size = self.val['d']['size']
-        #while i < size:
-        #    char = self.val['d']['data'][i]
-        #    if (char > 127):
-        #        ret += "\\u%x" % int(char)
-        #    else:
-        #        ret += chr(char)
-        #    i = i + 1
-        #return ret
         ret = ""
+        qt5 = 0
+        try:
+            # Qt4 has d->data, Qt5 doesn't.
+            self.val['d']['data']
+        except Exception:
+            qt5 = 1
+
         # The QString object might be not yet initialized. In this case size is a bogus value
         # and the following 2 lines might throw memory access error. Hence the try/catch.
         try:
-            dataAsCharPointer = self.val['d']['data'].cast(gdb.lookup_type("char").pointer())
+            if qt5:
+                dataAsCharPointer = (self.val['d'] + 1).cast(gdb.lookup_type("char").pointer())
+            else:
+                dataAsCharPointer = self.val['d']['data'].cast(gdb.lookup_type("char").pointer())
             ret = dataAsCharPointer.string(encoding = 'UTF-16', length = size * 2)
         except Exception:
             # swallow the exception and return empty string
