@@ -52,6 +52,9 @@ namespace Cpp {
 
 void setStaticMatchContext(QList<KDevelop::IndexedType> types);
 
+void executeSignalSlotCompletionItem( KTextEditor::Document* document, const KTextEditor::Range& enteredWord,
+                                      bool isSignal, const QString& name, const QString& signature );
+
 //A completion item used for completion of normal declarations while normal code-completion
 class NormalDeclarationCompletionItem : public KDevelop::NormalDeclarationCompletionItem {
 public:
@@ -71,6 +74,7 @@ public:
   bool completingTemplateParameters() const;
   
   mutable QString alternativeText; //Text shown when declaration is zero
+  mutable QString prefixText; //Text prepended to declaration
   
   //If this is true, alternativeText will be shown in the list, and will be inserted on execution.
   //Also the scope will be set to LocalScope when this attribute is true.
