@@ -46,7 +46,7 @@
 #include <language/duchain/functiondefinition.h>
 #include <interfaces/icore.h>
 #include <interfaces/iproject.h>
-#include "cppnewclass.h"
+//#include "cppnewclass.h"
 //#include "makeimplementationprivate.h"
 #include <templatedeclaration.h>
 #include "../cpputils.h"
@@ -57,6 +57,7 @@
 #include <language/duchain/types/functiontype.h>
 #include <kparts/mainwindow.h>
 #include <language/duchain/duchainutils.h>
+#include <language/duchain/classdeclaration.h>
 #include <dumptree.h>
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iselectioncontroller.h>
@@ -140,54 +141,119 @@ void SimpleRefactoring::doContextMenu(KDevelop::ContextMenuExtension& extension,
       }
     }
   }
-  if(ProjectItemContext* projectContext = dynamic_cast<ProjectItemContext*>(context)) {
-    //Actions on project-items
-    if(projectContext->items().count() == 1) {
-      ProjectBaseItem* item = projectContext->items().first();
-      if(item->folder() || item->target()) {
-        //Allow creating a class in the folder
-        QAction* action = new QAction(i18n  ("Create Class"), this);
-        action->setData(QVariant::fromValue<ProjectBaseItem*>(item));
-//         action->setIcon(KIcon("edit-rename"));
-        connect(action, SIGNAL(triggered(bool)), this, SLOT(executeNewClassAction()));
+}
 
-        extension.addAction(ContextMenuExtension::FileGroup, action);
+QString SimpleRefactoring::moveIntoSource(const IndexedDeclaration& iDecl)
+{
+  DUChainReadLocker lock;
+  Declaration* decl = iDecl.data();
+  if(!decl) {
+    return i18n("No declaration under cursor");
+  }
+
+  KDevelop::IndexedString url = decl->url();
+  KUrl targetUrl = decl->url().toUrl();
+  if(headerExtensions.contains(QFileInfo(targetUrl.toLocalFile()).suffix())) {
+    targetUrl = CppUtils::sourceOrHeaderCandidate(targetUrl);
+  }
+  if(!targetUrl.isValid()) {
+    ///@todo Create source file if it doesn't exist yet
+    return i18n("No source file available for %1.", targetUrl.prettyUrl());
+  }
+
+  const IndexedString indexedTargetUrl(targetUrl);
+
+  lock.unlock();
+  KDevelop::ReferencedTopDUContext top = DUChain::self()->waitForUpdate(url, KDevelop::TopDUContext::AllDeclarationsAndContexts);
+  KDevelop::ReferencedTopDUContext targetTopContext = DUChain::self()->waitForUpdate(indexedTargetUrl, KDevelop::TopDUContext::AllDeclarationsAndContexts);
+  lock.lock();
+
+  if(!targetTopContext) {
+    ///@todo Eventually create source file if it doesn't exist yet
+    return i18n("Failed to update DUChain for %1.", targetUrl.prettyUrl());
+  }
+
+  if(!top || !iDecl.data() || iDecl.data() != decl) {
+    return i18n("Declaration lost while updating.");
+  }
+
+  kDebug() << "moving" << decl->qualifiedIdentifier();
+  AbstractFunctionDeclaration* funDecl = dynamic_cast<AbstractFunctionDeclaration*>(decl);
+  FunctionType::Ptr funType = decl->type<FunctionType>();
+
+  if( !(decl->internalContext() && decl->internalContext()->type() == DUContext::Other
+      && funDecl && funDecl->internalFunctionContext() && funType) )
+  {
+    return i18n("Cannot create definition for this declaration.");
+  }
+
+  CodeRepresentation::Ptr code = createCodeRepresentation(decl->url());
+  if(!code) {
+    return i18n("No document for %1", decl->url().toUrl().prettyUrl());
+  }
+
+  SimpleRange headerRange = decl->internalContext()->rangeInCurrentRevision();
+  // remove whitespace in front of the header range
+  KTextEditor::Range prefixRange(funDecl->internalFunctionContext()->range().end.castToSimpleCursor().textCursor()
+                                  + KTextEditor::Cursor(0, 1) /* skip ) of function context */,
+                                 headerRange.start.textCursor());
+  const QString prefixText = code->rangeText(prefixRange);
+  for (int i = prefixText.length() - 1; i >= 0 && prefixText.at(i).isSpace(); --i) {
+    if (headerRange.start.column == 0) {
+      headerRange.start.line--;
+      if (headerRange.start.line == prefixRange.start().line()) {
+        headerRange.start.column = prefixRange.start().column() + i;
+      } else {
+        int lastNewline = prefixText.lastIndexOf('\n', i - 1);
+        headerRange.start.column = i - lastNewline - 1;
+        kWarning() << "UNSUPPORTED" << headerRange.start.column << lastNewline << i << prefixText;
       }
+    } else {
+      headerRange.start.column--;
     }
   }
-}
+  qDebug() << prefixText << prefixRange;
+  const QString body = code->rangeText(headerRange.textRange());
+  SourceCodeInsertion ins(targetTopContext);
+  QualifiedIdentifier namespaceIdentifier = decl->internalContext()->parentContext()->scopeIdentifier(false);
 
-void SimpleRefactoring::executeNewClassAction() {
-  QAction* action = qobject_cast<QAction*>(sender());
-  if(action) {
-    ProjectBaseItem* item = action->data().value<ProjectBaseItem*>();
-    createNewClass(item);
-  }else{
-    kWarning() << "strange problem";
+  ins.setSubScope(namespaceIdentifier);
+
+  QList<SourceCodeInsertion::SignatureItem> signature;
+
+  foreach(Declaration* argument,  funDecl->internalFunctionContext()->localDeclarations()) {
+    SourceCodeInsertion::SignatureItem item;
+    item.name = argument->identifier().toString();
+    item.type = argument->abstractType();
+    signature.append(item);
   }
-}
 
-void SimpleRefactoring::createNewClass(ProjectBaseItem* item)
-{
-  ///TODO: refactor and put this also into the CppNewClassAssistant
-  KUrl u;
-  
-  //Pick a folder to guess Possible URL for new class
-  if(item) {
-    ProjectFolderItem* ff = item->folder();
-    if(!ff && item->target())
-      ff=static_cast<ProjectBaseItem*>(item->parent())->folder();
-    
-    if(ff)
-      u=ff->url();
+  kDebug() << "qualified id:" << decl->qualifiedIdentifier() << "from mid:" << decl->qualifiedIdentifier().mid(namespaceIdentifier.count()) << namespaceIdentifier.count();
+
+  Identifier id(IndexedString(decl->qualifiedIdentifier().mid(namespaceIdentifier.count()).toString()));
+  kDebug() << "id:" << id;
+
+  if(!ins.insertFunctionDeclaration(id, funType->returnType(), signature, funType->modifiers() & AbstractType::ConstModifier, body)) {
+    return i18n("Insertion failed");
+  }
+  lock.unlock();
+  DocumentChangeSet::ChangeResult applied = ins.changes().applyAllChanges();
+  if(!applied) {
+    return i18n("Applying changes failed: %1", applied.m_failureReason);
   }
-  else
-    u = folderFromSelection();
 
-  //Run assistant
-  CppNewClass newClassGenerator(item);
-  CppNewClassAssistant newClassAssistant(qApp->activeWindow(), &newClassGenerator, u);
-  newClassAssistant.exec();
+  // replace header function body with a semicolon
+  DocumentChangeSet changeHeader;
+  changeHeader.addChange(DocumentChange(decl->url(), headerRange, body, ";"));
+  applied = changeHeader.applyAllChanges();
+  if(!applied) {
+    return i18n("Applying changes failed: %1", applied.m_failureReason);
+  }
+
+  ICore::self()->languageController()->backgroundParser()->addDocument(url);
+  ICore::self()->languageController()->backgroundParser()->addDocument(indexedTargetUrl);
+
+  return QString();
 }
 
 void SimpleRefactoring::executeMoveIntoSourceAction() {
@@ -196,96 +262,11 @@ void SimpleRefactoring::executeMoveIntoSourceAction() {
     IndexedDeclaration iDecl = action->data().value<IndexedDeclaration>();
     if(!iDecl.isValid())
       iDecl = declarationUnderCursor(false);
-    
-    DUChainReadLocker lock(DUChain::lock());
-    Declaration* decl = iDecl.data();
-    if(!decl) {
-      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("No declaration under cursor"));
-      return;
-    }
-    
-    KDevelop::IndexedString url = decl->url();
-    KUrl targetUrl = decl->url().toUrl();
-    if(headerExtensions.contains(QFileInfo(targetUrl.toLocalFile()).suffix())) {
-      targetUrl = CppUtils::sourceOrHeaderCandidate(targetUrl);
-    }
-    if(!targetUrl.isValid()) {
-      ///@todo Create source file if it doesn't exist yet
-      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("No source file available for %1.", targetUrl.prettyUrl()));
-      return;
-    }
 
-    
-    lock.unlock();
-    KDevelop::ReferencedTopDUContext top = DUChain::self()->waitForUpdate(url, KDevelop::TopDUContext::AllDeclarationsAndContexts);
-    KDevelop::ReferencedTopDUContext targetTopContext = DUChain::self()->waitForUpdate(IndexedString(targetUrl), KDevelop::TopDUContext::AllDeclarationsAndContexts);
-    lock.lock();
-    
-    if(!targetTopContext) {
-      ///@todo Eventually create source file if it doesn't exist yet
-      lock.unlock();
-      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Failed to update DU chain for %1.", targetUrl.prettyUrl()));
-      return;
+    const QString error = moveIntoSource(iDecl);
+    if (!error.isEmpty()) {
+      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), error);
     }
-    kDebug() << "moving" << decl->qualifiedIdentifier();
-    AbstractFunctionDeclaration* funDecl = dynamic_cast<AbstractFunctionDeclaration*>(decl);
-    FunctionType::Ptr funType = decl->type<FunctionType>();
-
-    if(top && iDecl.data() && iDecl.data() == decl)
-    {
-      if( !(decl->internalContext() && decl->internalContext()->type() == DUContext::Other && funDecl && funDecl->internalFunctionContext() && funType) ) {
-        lock.unlock();
-        KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Cannot create definition for this declaration."));
-        return;
-      }
-      
-      KDevelop::IDocument* doc = ICore::self()->documentController()->documentForUrl(decl->url().toUrl());
-      if(doc && doc->textDocument()) {
-        QString body = doc->textDocument()->text(decl->internalContext()->rangeInCurrentRevision().textRange());
-        SourceCodeInsertion ins(targetTopContext);
-        QualifiedIdentifier namespaceIdentifier = decl->internalContext()->parentContext()->scopeIdentifier(false);
-
-        ins.setSubScope(namespaceIdentifier);
-        
-        QList<SourceCodeInsertion::SignatureItem> signature;
-        
-        foreach(Declaration* argument,  funDecl->internalFunctionContext()->localDeclarations()) {
-          SourceCodeInsertion::SignatureItem item;
-          item.name = argument->identifier().toString();
-          item.type = argument->abstractType();
-          signature.append(item);
-        }
-        
-        kDebug() << "qualified id:" << decl->qualifiedIdentifier() << "from mid:" << decl->qualifiedIdentifier().mid(namespaceIdentifier.count()) << namespaceIdentifier.count();
-        
-        Identifier id(IndexedString(decl->qualifiedIdentifier().mid(namespaceIdentifier.count()).toString()));
-        kDebug() << "id:" << id;
-        
-        if(!ins.insertFunctionDeclaration(id, funType->returnType(), signature, funType->modifiers() & AbstractType::ConstModifier, body)) {
-          lock.unlock();
-          KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Insertion failed"));
-          return;
-        }
-        KTextEditor::Range range = decl->internalContext()->rangeInCurrentRevision().textRange();
-        lock.unlock();
-        if(!ins.changes().applyAllChanges()) {
-          KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Applying changes failed"));
-          return;
-        }
-        
-        doc->textDocument()->replaceText(range, QString(";"));
-        ICore::self()->languageController()->backgroundParser()->addDocument(url.toUrl());
-        ICore::self()->languageController()->backgroundParser()->addDocument(targetUrl);
-        
-      }else{
-        lock.unlock();
-        KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("No document for %1", decl->url().toUrl().prettyUrl()));
-      }
-    }else{
-      lock.unlock();
-      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Declaration lost while updating"));
-    }
-    
   }else{
     kWarning() << "strange problem";
   }
@@ -320,24 +301,87 @@ class SimpleRefactoringCollector : public KDevelop::UsesWidget::UsesWidgetCollec
   QVector<IndexedTopDUContext> m_allUsingContexts;
 };
 
-DocumentChangeSet::ChangeResult applyChangesToDeclarations(QString oldName, QString newName, DocumentChangeSet& changes, QList<IndexedDeclaration> declarations) {
+QPair<QString, QString> splitFileAtExtension(const QString& fileName)
+{
+  int idx = fileName.indexOf('.');
+  if (idx == -1) {
+    return qMakePair(fileName, QString());
+  }
+  return qMakePair(fileName.left(idx), fileName.mid(idx));
+}
+
+bool SimpleRefactoring::shouldRenameFile(Declaration* declaration)
+{
+  // only try to rename files when we renamed a class/struct
+  if (!dynamic_cast<ClassDeclaration*>(declaration)) {
+    return false;
+  }
+  const KUrl currUrl = declaration->topContext()->url().toUrl();
+  const QString fileName = currUrl.fileName();
+  const QPair<QString, QString> nameExtensionPair = splitFileAtExtension(fileName);
+  // check whether we renamed something that is called like the document it lives in
+  return nameExtensionPair.first.compare(declaration->identifier().toString(), Qt::CaseInsensitive) == 0;
+}
+
+QString SimpleRefactoring::newFileName(const KUrl& current, const QString& newName)
+{
+  QPair<QString, QString> nameExtensionPair = splitFileAtExtension(current.fileName());
+  // if current file is lowercased, keep that
+  if (nameExtensionPair.first == nameExtensionPair.first.toLower()) {
+    return newName.toLower() + nameExtensionPair.second;
+  } else {
+    return newName + nameExtensionPair.second;
+  }
+}
+
+DocumentChangeSet::ChangeResult SimpleRefactoring::addRenameFileChanges(const KUrl& current,
+                                                                        const QString& newName,
+                                                                        DocumentChangeSet* changes)
+{
+  DocumentChangeSet::ChangeResult result = changes->addDocumentRenameChange(
+    IndexedString(current), IndexedString(newFileName(current, newName)));
+  if (!result) {
+    return result;
+  }
+
+  // check for implementation file
+  const KUrl otherFile = CppUtils::sourceOrHeaderCandidate(current);
+  if (otherFile.isValid()) {
+    // also rename this other file
+    result = changes->addDocumentRenameChange(
+        IndexedString(otherFile), IndexedString(newFileName(otherFile, newName)));
+    if(!result) {
+      return result;
+    }
+  }
+  return true;
+}
+
+DocumentChangeSet::ChangeResult applyChangesToDeclarations(const QString& oldName,
+                                                           const QString& newName,
+                                                           DocumentChangeSet& changes,
+                                                           const QList<IndexedDeclaration>& declarations)
+{
   foreach(const IndexedDeclaration &decl, declarations) {
-    if(!decl.data())
+    Declaration* declaration = decl.data();
+    if(!declaration) {
       continue;
-    TopDUContext* top = decl.data()->topContext();
-    if(decl.data()->range().isEmpty()) {
+    }
+    if (declaration->range().isEmpty()) {
       kDebug() << "found empty declaration";
     }
-    if(ClassFunctionDeclaration* classFun = dynamic_cast<ClassFunctionDeclaration*>(decl.data())) {
-      if(classFun->isDestructor()) {
-        //Our C++, destructors only have the range of the text behind the "~"
-//         newName = destructorForName(Identifier(newName)).identifier().str();
-//         oldName = destructorForName(Identifier(oldName)).identifier().str();
+    TopDUContext* top = declaration->topContext();
+    DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(top->url(), declaration->rangeInCurrentRevision(), oldName, newName));
+    if (!result) {
+      return result;
+    }
+
+    if (SimpleRefactoring::shouldRenameFile(declaration)) {
+      result = SimpleRefactoring::addRenameFileChanges(top->url().toUrl(), newName, &changes);
+      if (!result) {
+        return result;
       }
     }
-    DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(top->url(), decl.data()->rangeInCurrentRevision(), oldName, newName));
-    if(!result)
-      return result;
   }
   return DocumentChangeSet::ChangeResult(true);
 }
@@ -411,64 +455,67 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
 
   //Since we don't yet know what the text should be replaced with, we just collect the top-contexts to process
   SimpleRefactoringCollector* collector = new SimpleRefactoringCollector(declaration);
-
-  QDialog dialog;
-
-  QTabWidget tabWidget;
-
-  UsesWidget uses(declaration, collector);
+  UsesWidget* uses = new UsesWidget(declaration, collector);
 
   QWidget* navigationWidget = declaration->context()->createNavigationWidget(declaration);
   AbstractNavigationWidget* abstractNavigationWidget = dynamic_cast<AbstractNavigationWidget*>(navigationWidget);
 
   if(abstractNavigationWidget) { //So the context-links work
-    connect(&uses, SIGNAL(navigateDeclaration(KDevelop::IndexedDeclaration)), abstractNavigationWidget, SLOT(navigateDeclaration(KDevelop::IndexedDeclaration)));
+    connect(uses, SIGNAL(navigateDeclaration(KDevelop::IndexedDeclaration)),
+            abstractNavigationWidget, SLOT(navigateDeclaration(KDevelop::IndexedDeclaration)));
 //     connect(uses, SIGNAL(navigateDeclaration(IndexedDeclaration)), tabWidget, SLOT(setCurrentIndex(...)));
 ///@todo Switch the tab in the tab-widget, so the user will notice that the declaration is being shown
   }
 
-  QVBoxLayout verticalLayout;
-  QHBoxLayout actionsLayout;
-  dialog.setLayout(&verticalLayout);
-  dialog.setWindowTitle(i18n("Rename %1", declaration->toString()));
+  QScopedPointer<QDialog> dialog(new QDialog);
+  dialog->setWindowTitle(i18n("Rename %1", declaration->toString()));
 
-  QLabel newNameLabel(i18n("New name:"));
-  actionsLayout.addWidget(&newNameLabel);
+  QVBoxLayout* verticalLayout = new QVBoxLayout;
+  dialog->setLayout(verticalLayout);
 
-  QLineEdit edit(declaration->identifier().identifier().str());
-  newNameLabel.setBuddy(&edit);
+  QHBoxLayout* actionsLayout = new QHBoxLayout;
 
-  actionsLayout.addWidget(&edit);
-  edit.setText(originalName);
-  edit.setFocus();
-  edit.selectAll();
-  QPushButton goButton(i18n("Rename"));
-  goButton.setToolTip(i18n("Note: All overloaded functions, overloads, forward-declarations, etc. will be renamed too"));
-  actionsLayout.addWidget(&goButton);
-  connect(&goButton, SIGNAL(clicked(bool)), &dialog, SLOT(accept()));
+  QLabel* newNameLabel = new QLabel(i18n("New name:"));
+  actionsLayout->addWidget(newNameLabel);
 
-  QPushButton cancelButton(i18n("Cancel"));
-  actionsLayout.addWidget(&cancelButton);
-  verticalLayout.addLayout(&actionsLayout);
-
-  tabWidget.addTab(&uses, i18n("Uses"));
-  if(navigationWidget)
-    tabWidget.addTab(navigationWidget, i18n("Declaration Info"));
-
-  verticalLayout.addWidget(&tabWidget);
-
-  connect(&cancelButton, SIGNAL(clicked(bool)), &dialog, SLOT(reject()));
+  QLineEdit* edit = new QLineEdit(declaration->identifier().toString());
 
   lock.unlock();
-  dialog.resize( 750, 550 );
+  ///NOTE: do not access declaration anymore now!
 
-  if(dialog.exec() != QDialog::Accepted) {
+  newNameLabel->setBuddy(edit);
+  edit->setText(originalName);
+  edit->setFocus();
+  edit->selectAll();
+  actionsLayout->addWidget(edit);
+
+  QPushButton* goButton = new QPushButton(i18n("Rename"));
+  goButton->setToolTip(i18n("Note: All overloaded functions, overloads, forward-declarations, etc. will be renamed too"));
+  actionsLayout->addWidget(goButton);
+  connect(goButton, SIGNAL(clicked(bool)), dialog.data(), SLOT(accept()));
+
+  QPushButton* cancelButton = new QPushButton(i18n("Cancel"));
+  actionsLayout->addWidget(cancelButton);
+  verticalLayout->addLayout(actionsLayout);
+
+  QTabWidget* tabWidget = new QTabWidget;
+  verticalLayout->addWidget(tabWidget);
+  tabWidget->addTab(uses, i18n("Uses"));
+  if (navigationWidget) {
+    tabWidget->addTab(navigationWidget, i18n("Declaration Info"));
+  }
+
+  connect(cancelButton, SIGNAL(clicked(bool)), dialog.data(), SLOT(reject()));
+
+  dialog->resize( 750, 550 );
+
+  if(dialog->exec() != QDialog::Accepted) {
     kDebug() << "stopped";
     return;
   }
   //It would be nicer to scope this, but then "uses" would not survive
 
-  replacementName = edit.text();
+  replacementName = edit->text();
 
 
   if(replacementName == originalName || replacementName.isEmpty())
@@ -507,13 +554,22 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
     return;
   }
 
+  lock.unlock();
   ///We have to ignore failed changes for now, since uses of a constructor or of operator() may be created on "(" parens
   changes.setReplacementPolicy(DocumentChangeSet::IgnoreFailedChange);
   changes.setFormatPolicy(KDevelop::DocumentChangeSet::NoAutoFormat);
-  result = changes.applyAllChanges();
+
+  m_pendingChanges = changes;
+  ///NOTE: this is required, otherwise, if you rename a file it will crash...
+  QMetaObject::invokeMethod(this, "applyChangesDelayed", Qt::QueuedConnection);
+}
+
+void SimpleRefactoring::applyChangesDelayed()
+{
+  DocumentChangeSet::ChangeResult result = m_pendingChanges.applyAllChanges();
+  m_pendingChanges = DocumentChangeSet();
   if(!result) {
       KMessageBox::error(0, i18n("Applying changes failed: %1", result.m_failureReason));
-      return;
   }
 }
 
