@@ -312,7 +312,7 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
         }
 
     node->expression->ducontext = currentContext();
-    ExpressionParser parser(false, false, isDecltypeInParen);
+    ExpressionParser parser(false, false, isDecltypeInParen, m_mapAst);
     ExpressionEvaluationResult result = parser.evaluateType(node->expression, editor()->parseSession(), topContext());
     AbstractType::Ptr type = result.type.abstractType();
     // make reference for decltype in additional parens - but only if it's not already a reference
@@ -417,8 +417,9 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     closeType();
 }
 
-void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
-  if(m_onlyComputeSimplified) {
+void TypeBuilder::createIntegralTypeForExpression(ExpressionAST* expression)
+{
+  if (!expression) {
     return;
   }
 
@@ -430,6 +431,71 @@ void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
     }
   }
 
+  if (!integral) {
+    return;
+  }
+
+  if (!(integral->modifiers() & AbstractType::ConstModifier) && !m_lastTypeWasAuto) {
+    return;
+  }
+
+  bool openedType = false;
+  bool delay = false;
+  ///@todo This is nearly a copy of visitEnumerator and parts of visitSimpleTypeSpecifier, merge it
+  {
+    //Parse the expression, and create a CppConstantIntegralType, since we know the value
+    Cpp::ExpressionParser parser;
+    DUChainReadLocker lock(DUChain::lock());
+    expression->ducontext = currentContext();
+    Cpp::ExpressionEvaluationResult res = parser.evaluateType( expression, editor()->parseSession() );
+
+    //Delay the type-resolution of template-parameters
+    if( res.allDeclarations.size() ) {
+      Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
+      ///@todo Do a check on all involved types, also template parameters, by giving the parameter to evaluateType
+      if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
+        delay = true;
+    }
+
+    if ( !delay && res.isValid() && res.isInstance ) {
+      AbstractType::Ptr type = res.type.abstractType();
+      if ( m_lastTypeWasAuto )
+      {
+        // remove references or aliases
+        type = TypeUtils::realType( type, topContext() );
+        // Turn "5" into "int"
+        type = TypeUtils::removeConstants( type, topContext() );
+        // ensure proper const modifier is set
+        type->setModifiers( integral->modifiers() );
+        if (ReferenceType::Ptr ref = lastType().cast<ReferenceType>()) {
+          ref->setBaseType( type );
+          type = ref.cast<AbstractType>();
+        }
+      }
+
+      openType( type );
+      openedType = true;
+    }
+  }
+  if( delay || !openedType ) {
+    ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
+    QString str = stringFromSessionTokens( editor()->parseSession(), expression->start_token, expression->end_token ).trimmed();
+
+    QualifiedIdentifier id( str, true );
+
+    openDelayedType(IndexedTypeIdentifier(id), expression, DelayedType::Delayed);
+    openedType = true;
+  }
+
+  if(openedType)
+    closeType();
+}
+
+void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
+  if(m_onlyComputeSimplified) {
+    return;
+  }
+
   ExpressionAST* expression = 0;
   if (node->initializer_clause && node->initializer_clause->expression) {
     // auto foo = ...;
@@ -439,63 +505,16 @@ void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
     expression = node->expression;
   }
 
-  if(integral && (integral->modifiers() & AbstractType::ConstModifier || m_lastTypeWasAuto)
-      && expression)
-  {
-    //Parse the expression, and create a CppConstantIntegralType, since we know the value
-    Cpp::ExpressionParser parser;
-
-    bool openedType = false;
-    Cpp::ExpressionEvaluationResult res;
-
-    bool delay = false;
-    ///@todo This is nearly a copy of visitEnumerator and parts of visitSimpleTypeSpecifier, merge it
-    if(!delay) {
-      DUChainReadLocker lock(DUChain::lock());
-      expression->ducontext = currentContext();
-      res = parser.evaluateType( expression, editor()->parseSession() );
-
-      //Delay the type-resolution of template-parameters
-      if( res.allDeclarations.size() ) {
-        Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
-        ///@todo Do a check on all involved types, also template parameters, by giving the parameter to evaluateType
-        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
-          delay = true;
-      }
-
-      if ( !delay && res.isValid() && res.isInstance ) {
-        AbstractType::Ptr type = res.type.abstractType();
-        if ( m_lastTypeWasAuto )
-        {
-          type->setModifiers( integral->modifiers() );
-          // Turn "5" into "int"
-          type = TypeUtils::removeConstants( type, topContext() );
-          if (ReferenceType::Ptr ref = lastType().cast<ReferenceType>()) {
-            ref->setBaseType( type );
-            type = ref.cast<AbstractType>();
-          }
-        }
-        
-        openType( type );
-        openedType = true;
-      }
-    }
-    if( delay || !openedType ) {
-      ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
-      QString str = stringFromSessionTokens( editor()->parseSession(), expression->start_token, expression->end_token ).trimmed();
-
-      QualifiedIdentifier id( str, true );
-
-      openDelayedType(IndexedTypeIdentifier(id), node, DelayedType::Delayed);
-      openedType = true;
-    }
-    
-    if(openedType)
-      closeType();
-  }
+  createIntegralTypeForExpression(expression);
 }
 
-void TypeBuilder::closeTypeForInitializer(InitializerAST* /*node*/) {
+void TypeBuilder::createTypeForCondition(ConditionAST* node)
+{
+  if (m_onlyComputeSimplified) {
+    return;
+  }
+
+  createIntegralTypeForExpression(node->expression);
 }
 
 bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass) {
@@ -742,17 +761,18 @@ void TypeBuilder::createTypeForDeclarator(DeclaratorAST *node) {
     } while (it != end);
   }
 
-  if (node->parameter_declaration_clause)
+  if (node->parameter_declaration_clause) {
     // New function type
     openType(FunctionType::Ptr(openFunction(node)));
+  }
 }
 
 void TypeBuilder::closeTypeForDeclarator(DeclaratorAST *node) {
-  if (node->parameter_declaration_clause)
+  if (node->parameter_declaration_clause) {
     closeType();
+  }
 }
 
-
 void TypeBuilder::visitArrayExpression(ExpressionAST* expression)
 {
   if(m_onlyComputeSimplified) {
