@@ -469,6 +469,8 @@ public:
   ExceptionSpecificationAST *exception_spec;
   TrailingReturnTypeAST* trailing_return_type;
   const ListNode<uint> *virt_specifiers;
+  // either & or && token or null
+  uint ref_qualifier;
   bool isVariadic;
 };
 
@@ -819,8 +821,8 @@ public:
 
   DECLARE_AST_NODE(Namespace)
 
-  uint namespace_name;
   LinkageBodyAST *linkage_body;
+  uint namespace_name;
   bool inlined : 1;
 };
 
@@ -950,11 +952,21 @@ public:
 
   DECLARE_AST_NODE(PrimaryExpression)
 
-  StringLiteralAST *literal;
-  uint token;
-  StatementAST *expression_statement;
-  ExpressionAST *sub_expression;
-  NameAST *name;
+  union {
+    StringLiteralAST *literal;
+    StatementAST *expression_statement;
+    ExpressionAST *sub_expression;
+    NameAST *name;
+    uint token;
+  };
+  enum Type {
+    Literal,
+    Token,
+    Statement,
+    SubExpression,
+    Name
+  };
+  Type type;
 };
 
 class PtrOperatorAST : public AST
@@ -1208,11 +1220,11 @@ public:
 
   DECLARE_AST_NODE(TypeParameter)
 
-  uint type;
   NameAST *name;
   TypeIdAST *type_id;
   const ListNode<TemplateParameterAST*> *template_parameters;
   NameAST *template_name;
+  uint type;
   bool isVariadic;
 };
 
@@ -1242,9 +1254,9 @@ public:
 
   DECLARE_AST_NODE(UnqualifiedName)
 
-  uint tilde;
+  bool tilde : 1;
+  bool ellipsis : 1;
   uint id;
-  bool ellipsis;
   OperatorFunctionIdAST *operator_id;
   const ListNode<TemplateArgumentAST*> *template_arguments;
 };
