@@ -1622,9 +1622,11 @@ int CMakeProjectVisitor::visit(const GetFilenameComponentAst *filecomp)
     QString val;
     switch(filecomp->type())
     {
-        case GetFilenameComponentAst::Path:
-            val=fi.path();
-            break;
+        case GetFilenameComponentAst::Path: {
+            int idx = filecomp->fileName().lastIndexOf(QDir::separator());
+            if(idx>=0)
+                val=filecomp->fileName().left(idx);
+        }   break;
         case GetFilenameComponentAst::Absolute:
             val=fi.absoluteFilePath();
             break;
@@ -1698,8 +1700,12 @@ int CMakeProjectVisitor::visit(const ListAst *list)
             m_vars->insert(list->list(), theList);
             break;
         case ListAst::Find: {
-            int idx=theList.indexOf(list->elements().first());
-            
+            QString element;
+            int idx=-1;
+            if(!list->elements().isEmpty()) {
+                element = list->elements().first();
+                idx=theList.indexOf(element);
+            }
             m_vars->insert(list->output(), QStringList(QString::number(idx)));
             kDebug(9042) << "List: Find" << theList << list->output() << list->elements() << idx;
         }   break;
