@@ -18,11 +18,9 @@
 */
 
 #include "cmakemanagertest.h"
+#include "testhelpers.h"
 #include "cmakemodelitems.h"
-#include "cmake-test-paths.h"
-
 #include <icmakemanager.h>
-#include <cmakebuilddirchooser.h>
 
 #include <qtest.h>
 #include <qtest_kde.h>
@@ -43,83 +41,6 @@ QTEST_KDEMAIN(CMakeManagerTest, GUI )
     QVERIFY2(gotSignal, "Timeout while waiting for opened signal");\
 } void(0)
 
-static QString currentBuildDirKey = "CurrentBuildDir";
-static QString currentCMakeBinaryKey = "Current CMake Binary";
-static QString currentBuildTypeKey = "CurrentBuildType";
-static QString currentInstallDirKey = "CurrentInstallDir";
-static QString currentExtraArgumentsKey = "Extra Arguments";
-static QString projectRootRelativeKey = "ProjectRootRelative";
-static QString projectBuildDirs = "BuildDirs";
-
-struct TestProjectPaths {
-    // foo/
-    KUrl sourceDir;
-    // foo/foo.kdev4
-    KUrl projectFile;
-    // foo/.kdev4/foo.kdev4
-    KUrl configFile;
-};
-
-TestProjectPaths projectPaths(const QString& project, QString name = QString())
-{
-    if (name.isEmpty()) {
-        name = project;
-    }
-
-    TestProjectPaths paths;
-    QFileInfo info(CMAKE_TESTS_PROJECTS_DIR "/" + project);
-    Q_ASSERT(info.exists());
-
-    paths.sourceDir = info.canonicalFilePath();
-    paths.sourceDir.adjustPath(KUrl::AddTrailingSlash);
-
-    paths.projectFile = paths.sourceDir;
-    paths.projectFile.addPath(name + ".kdev4");
-    Q_ASSERT(QFile::exists(paths.projectFile.toLocalFile()));
-
-    paths.configFile = paths.sourceDir;
-    paths.configFile.addPath(".kdev4/" + name + ".kdev4");
-
-    return paths;
-}
-
-/**
- * apply default configuration to project in @p sourceDir called @p projectName
- * 
- * this prevents the dialog to popup asking for user interaction
- * which should never happen in an automated unit test
- */
-void defaultConfigure(const TestProjectPaths& paths)
-{
-    KConfig config(paths.configFile.toLocalFile());
-    // clear config
-    config.deleteGroup("CMake");
-
-    // apply default configuration
-    CMakeBuildDirChooser bd;
-    bd.setSourceFolder(paths.sourceDir);
-    // we don't want to execute, just pick the defaults from the dialog
-
-    KConfigGroup cmakeGrp = config.group("CMake");
-    {
-        QDir buildFolder( bd.buildFolder().toLocalFile() );
-        if ( !buildFolder.exists() ) {
-            if ( !buildFolder.mkpath( buildFolder.absolutePath() ) ) {
-                QFAIL("The build directory did not exist and could not be created.");
-            }
-        }
-    }
-
-    cmakeGrp.writeEntry( currentBuildDirKey, bd.buildFolder() );
-    cmakeGrp.writeEntry( currentCMakeBinaryKey, bd.cmakeBinary() );
-    cmakeGrp.writeEntry( currentInstallDirKey, bd.installPrefix() );
-    cmakeGrp.writeEntry( currentExtraArgumentsKey, bd.extraArguments() );
-    cmakeGrp.writeEntry( currentBuildTypeKey, bd.buildType() );
-    cmakeGrp.writeEntry( projectBuildDirs, QStringList() << bd.buildFolder().toLocalFile());
-
-    config.sync();
-}
-
 using namespace KDevelop;
 
 void CMakeManagerTest::initTestCase()
@@ -280,3 +201,30 @@ void CMakeManagerTest::testTargetIncludePaths()
     }
     QVERIFY(foundInTarget);
 }
+
+void CMakeManagerTest::testConditionsInSubdirectoryBasedOnRootVariables()
+{
+    const TestProjectPaths paths = projectPaths("conditions_in_subdirectory_based_on_root_variables");
+    defaultConfigure(paths);
+
+    ICore::self()->projectController()->openProject(paths.projectFile);
+
+    WAIT_FOR_OPEN_SIGNAL;
+
+    IProject* project = ICore::self()->projectController()->findProjectByName("conditions_in_subdirectory_based_on_root_variables");
+    QVERIFY(project);
+    QVERIFY(project->buildSystemManager());
+
+    QCOMPARE(paths.projectFile, project->projectFileUrl());
+    QCOMPARE(paths.sourceDir, project->folder());
+
+    KUrl rootFooCpp(paths.sourceDir, "foo.cpp");
+    QVERIFY(QFile::exists(rootFooCpp.toLocalFile()));
+    QList< ProjectBaseItem* > rootFooItems = project->itemsForUrl(rootFooCpp);
+    QCOMPARE(rootFooItems.size(), 4); // three items for the targets, one item for the plain file
+
+    KUrl subdirectoryFooCpp(paths.sourceDir, "subdirectory/foo.cpp");
+    QVERIFY(QFile::exists(subdirectoryFooCpp.toLocalFile()));
+    QList< ProjectBaseItem* > subdirectoryFooItems = project->itemsForUrl(subdirectoryFooCpp);
+    QCOMPARE(subdirectoryFooItems.size(), 4); // three items for the targets, one item for the plain file
+}
